@@ -6,7 +6,7 @@
 use v5.26;
 use Object::Pad 0.35;
 
-package Device::Chip::Adapter 0.16;
+package Device::Chip::Adapter 0.17;
 role Device::Chip::Adapter :repr(HASH) :compat(invokable);
 
 use utf8;
@@ -493,6 +493,19 @@ chip then reads the give number of bytes back, returning them. These two
 operations must be performed within a single I²C transaction using a repeated
 start condition.
 
+=head2 txn
+
+   $result = await $i2c->txn( async sub { ... } );
+
+Performs a complete custom I²C transaction. Within the code block invoked by
+the transaction, the C<write> and C<read> methods may be called, but they will
+B<not> cause stop conditions to be sent on the wire. A stop condition will be
+sent after the code has finished. The return value from this method will be
+whatever is returned by the inner code block.
+
+This method acts as a mutex lock, ensuring only one transaction can run
+concurrently.
+
 =cut
 
 =head1 UART PROTOCOL
